@@ -1,3 +1,3 @@
 from .base import Encrypter
 
-__all__ = ['Encrypter']
+__all__ = ["Encrypter"]
